@@ -8,6 +8,9 @@ import pandas as pd
 from . import lint, core
 from .C import *
 
+__all__ = ['get_condition_df', 'write_condition_df', 'create_condition_df',
+           'get_parametric_overrides']
+
 
 def get_condition_df(
         condition_file: Union[str, pd.DataFrame, None]
@@ -48,8 +51,7 @@ def write_condition_df(df: pd.DataFrame, filename: str) -> None:
         df: PEtab condition table
         filename: Destination file name
     """
-    with open(filename, 'w') as fh:
-        df.to_csv(fh, sep='\t', index=True)
+    df.to_csv(filename, sep='\t', index=True)
 
 
 def create_condition_df(parameter_ids: Iterable[str],
@@ -61,8 +63,8 @@ def create_condition_df(parameter_ids: Iterable[str],
         parameter_ids: the columns
         condition_ids: the rows
     Returns:
-        A ``pandas.DataFrame`` with empty given rows and columns and all nan
-        values
+        A :py:class:`pandas.DataFrame` with empty given rows and columns and
+        all nan values
     """
 
     condition_ids = [] if condition_ids is None else list(condition_ids)
