@@ -13,6 +13,11 @@ from . import yaml
 from .C import *  # noqa: F403
 
 logger = logging.getLogger(__name__)
+__all__ = ['get_simulation_df', 'write_simulation_df', 'get_visualization_df',
+           'write_visualization_df', 'get_notnull_columns',
+           'get_observable_id', 'flatten_timepoint_specific_output_overrides',
+           'concat_tables', 'to_float_if_float', 'is_empty',
+           'create_combine_archive', 'unique_preserve_order']
 
 
 def get_simulation_df(simulation_file: str) -> pd.DataFrame:
@@ -35,8 +40,7 @@ def write_simulation_df(df: pd.DataFrame, filename: str) -> None:
         df: PEtab simulation table
         filename: Destination file name
     """
-    with open(filename, 'w') as fh:
-        df.to_csv(fh, sep='\t', index=False)
+    df.to_csv(filename, sep='\t', index=False)
 
 
 def get_visualization_df(visualization_file: str) -> pd.DataFrame:
@@ -65,8 +69,7 @@ def write_visualization_df(df: pd.DataFrame, filename: str) -> None:
         df: PEtab visualization table
         filename: Destination file name
     """
-    with open(filename, 'w') as fh:
-        df.to_csv(fh, sep='\t', index=False)
+    df.to_csv(filename, sep='\t', index=False)
 
 
 def get_notnull_columns(df: pd.DataFrame, candidates: Iterable):
@@ -232,13 +235,14 @@ def is_empty(val) -> bool:
         val: The value to check.
 
     Returns:
-        empty: Whether the field is to be considered empty.
+        Whether the field is to be considered empty.
     """
     return val == '' or pd.isnull(val)
 
 
 def create_combine_archive(
-        yaml_file: str, filename: str,
+        yaml_file: str,
+        filename: str,
         family_name: Optional[str] = None,
         given_name: Optional[str] = None,
         email: Optional[str] = None,
@@ -249,6 +253,7 @@ def create_combine_archive(
 
     Arguments:
         yaml_file: Path to PEtab YAML file
+        filename: Destination file name
         family_name: Family name of archive creator
         given_name: Given name of archive creator
         email: E-mail address of archive creator
@@ -355,7 +360,8 @@ def unique_preserve_order(seq: Sequence) -> List:
     """Return a list of unique elements in Sequence, keeping only the first
     occurrence of each element
 
-    seq: Sequence to prune
+    Parameters:
+        seq: Sequence to prune
 
     Returns:
         List of unique elements in ``seq``
