@@ -17,6 +17,36 @@ from . import (core, parameters, sbml, measurements)
 from .C import *  # noqa: F403
 
 logger = logging.getLogger(__name__)
+__all__ = ['assert_all_parameters_present_in_parameter_df',
+           'assert_measured_observables_defined',
+           'assert_measurement_conditions_present_in_condition_table',
+           'assert_measurements_not_null',
+           'assert_measurements_numeric',
+           'assert_model_parameters_in_condition_or_parameter_table',
+           'assert_no_leading_trailing_whitespace',
+           'assert_noise_distributions_valid',
+           'assert_parameter_bounds_are_numeric',
+           'assert_parameter_estimate_is_boolean',
+           'assert_parameter_id_is_string',
+           'assert_parameter_prior_parameters_are_valid',
+           'assert_parameter_prior_type_is_valid',
+           'assert_parameter_scale_is_valid',
+           'assert_unique_observable_ids',
+           'assert_unique_parameter_ids',
+           'check_condition_df',
+           'check_ids',
+           'check_measurement_df',
+           'check_observable_df',
+           'check_parameter_bounds',
+           'check_parameter_df',
+           'condition_table_is_parameter_free',
+           'get_non_unique',
+           'is_scalar_float',
+           'is_valid_identifier',
+           'lint_problem',
+           'measurement_table_has_observable_parameter_numeric_overrides',
+           'measurement_table_has_timepoint_specific_mappings',
+           'observable_table_has_nontrivial_noise_formula']
 
 
 def _check_df(df: pd.DataFrame, req_cols: Iterable, name: str) -> None:
@@ -149,6 +179,9 @@ def check_measurement_df(df: pd.DataFrame,
         measurements.assert_overrides_match_parameter_count(
             df, observable_df)
 
+    assert_measurements_not_null(df)
+    assert_measurements_numeric(df)
+
 
 def check_parameter_df(
         df: pd.DataFrame,
@@ -307,8 +340,8 @@ def assert_all_parameters_present_in_parameter_df(
 def assert_measured_observables_defined(
         measurement_df: pd.DataFrame,
         observable_df: pd.DataFrame) -> None:
-    """Check if all observables in the measurement table have been defined in the
-    observable table
+    """Check if all observables in the measurement table have been defined in
+    the observable table
 
     Arguments:
         measurement_df: PEtab measurement table
@@ -336,8 +369,8 @@ def condition_table_is_parameter_free(condition_df: pd.DataFrame) -> bool:
         condition_df: PEtab condition table
 
     Returns:
-        True if there are no parameter overrides in the condition table,
-        False otherwise.
+        ``True`` if there are no parameter overrides in the condition table,
+        ``False`` otherwise.
     """
 
     return len(petab.get_parametric_overrides(condition_df)) == 0
@@ -448,7 +481,7 @@ def assert_parameter_prior_type_is_valid(
         parameter_df: PEtab parameter table
 
     Raises:
-        AssertionError in case of invalid prior
+        AssertionError: in case of invalid prior
     """
     for col in [INITIALIZATION_PRIOR_TYPE, OBJECTIVE_PRIOR_TYPE]:
         if col not in parameter_df.columns:
@@ -468,7 +501,7 @@ def assert_parameter_prior_parameters_are_valid(
         parameter_df: PEtab parameter table
 
     Raises:
-        AssertionError in case of invalide prior parameters
+        AssertionError: in case of invalid prior parameters
     """
     prior_type_cols = [INITIALIZATION_PRIOR_TYPE,
                        OBJECTIVE_PRIOR_TYPE]
@@ -530,10 +563,11 @@ def is_scalar_float(x: Any):
     """
     Checks whether input is a number or can be transformed into a number
     via float
+
     :param x:
         input
     :return:
-        True if is or can be converted to number, False otherwise.
+        ``True`` if is or can be converted to number, ``False`` otherwise.
     """
     if isinstance(x, numbers.Number):
         return True
@@ -545,7 +579,7 @@ def is_scalar_float(x: Any):
 
 
 def measurement_table_has_timepoint_specific_mappings(
-        measurement_df: pd.DataFrame,
+        measurement_df: Optional[pd.DataFrame],
         allow_scalar_numeric_noise_parameters: bool = False,
         allow_scalar_numeric_observable_parameters: bool = False,
 ) -> bool:
@@ -558,16 +592,19 @@ def measurement_table_has_timepoint_specific_mappings(
             PEtab measurement table
 
         allow_scalar_numeric_noise_parameters:
-            ignore scalar numeric assignments to noiseParamater placeholders
+            ignore scalar numeric assignments to noiseParameter placeholders
 
         allow_scalar_numeric_observable_parameters:
-            ignore scalar numeric assignments to observableParamater
+            ignore scalar numeric assignments to observableParameter
             placeholders
 
     Returns:
         True if there are time-point or replicate specific (non-numeric)
         parameter assignments in the measurement table, False otherwise.
     """
+    if measurement_df is None:
+        return False
+
     # since we edit it, copy it first
     measurement_df = copy.deepcopy(measurement_df)
 
@@ -601,7 +638,6 @@ def measurement_table_has_timepoint_specific_mappings(
          SIMULATION_CONDITION_ID,
          PREEQUILIBRATION_CONDITION_ID])
     grouped_df2 = measurement_df.groupby(grouping_cols)
-
     # data frame has timepoint specific overrides if grouping by noise
     # parameters and observable parameters in addition to observable,
     # condition and preeq id yields more groups
@@ -609,7 +645,7 @@ def measurement_table_has_timepoint_specific_mappings(
 
 
 def observable_table_has_nontrivial_noise_formula(
-        observable_df: pd.DataFrame) -> bool:
+        observable_df: Optional[pd.DataFrame]) -> bool:
     """
     Does any observable have a noise formula that is not just a single
     parameter?
@@ -618,9 +654,11 @@ def observable_table_has_nontrivial_noise_formula(
         observable_df: PEtab observable table
 
     Returns:
-        True if any noise formula does not consist of a single identifier,
-        False otherwise.
+        ``True`` if any noise formula does not consist of a single identifier,
+        ``False`` otherwise.
     """
+    if observable_df is None:
+        return False
 
     return not observable_df[NOISE_FORMULA].apply(
         lambda x: is_scalar_float(x) or
@@ -636,8 +674,8 @@ def measurement_table_has_observable_parameter_numeric_overrides(
         measurement_df: PEtab measurement table
 
     Returns:
-        True if there are any numbers to override observable/noise parameters,
-        False otherwise.
+        ``True`` if there are any numbers to override observable/noise
+        parameters, ``False`` otherwise.
     """
     if OBSERVABLE_PARAMETERS not in measurement_df:
         return False
@@ -711,7 +749,7 @@ def lint_problem(problem: 'petab.Problem') -> bool:
         problem: PEtab problem to check
 
     Returns:
-        True is errors occurred, False otherwise
+        ``True`` if errors occurred, ``False`` otherwise
     """
     # pylint: disable=too-many-statements
     errors_occurred = False
@@ -873,6 +911,51 @@ def assert_measurement_conditions_present_in_condition_table(
                              + str(missing_conditions))
 
 
+def assert_measurements_not_null(
+    measurement_df: pd.DataFrame,
+) -> None:
+    """Check whether all measurements are not null.
+
+    Arguments:
+        measurement_df:
+            PEtab measurement table.
+
+    Raises:
+        AssertionError:
+            Some measurement value(s) are null (missing).
+    """
+    if measurement_df[MEASUREMENT].isnull().any():
+        raise AssertionError('Some measurement(s) are null (missing).')
+
+
+def assert_measurements_numeric(
+    measurement_df: pd.DataFrame,
+) -> None:
+    """Check whether all measurements are numeric.
+
+    Note that null (missing) measurements are ignored.
+
+    Arguments:
+        measurement_df:
+            PEtab measurement table.
+
+    Raises:
+        AssertionError:
+            Some measurement value(s) are not numeric.
+    """
+    not_null_measurement_values = measurement_df[MEASUREMENT].dropna()
+    all_measurements_are_numeric = (
+        pd.to_numeric(not_null_measurement_values, errors='coerce')
+        .notnull()
+        .all()
+    )
+    if not all_measurements_are_numeric:
+        raise AssertionError(
+            'Some values in the `petab.C.MEASUREMENT` column of the PEtab '
+            'measurements table are not numeric.'
+        )
+
+
 def is_valid_identifier(x: str) -> bool:
     """Check whether `x` is a valid identifier
 
@@ -900,7 +983,7 @@ def check_ids(ids: Iterable[str], kind: str = '') -> None:
         kind: Kind of IDs, for more informative error message
 
     Raises:
-        ValueError - in case of invalid IDs
+        ValueError: in case of invalid IDs
     """
     invalids = [(index, _id)
                 for index, _id in enumerate(ids)
