@@ -1,6 +1,6 @@
 """Functions for working with the PEtab observables table"""
 
-from _collections import OrderedDict
+from collections import OrderedDict
 from typing import Union, List
 
 import libsbml
@@ -11,6 +11,13 @@ import sympy as sp
 from . import lint, core
 from .C import *  # noqa: F403
 
+__all__ = ['create_observable_df',
+           'get_formula_placeholders',
+           'get_observable_df',
+           'get_output_parameters',
+           'get_placeholders',
+           'write_observable_df']
+
 
 def get_observable_df(
         observable_file: Union[str, pd.DataFrame, None]
@@ -53,8 +60,7 @@ def write_observable_df(df: pd.DataFrame, filename: str) -> None:
         df: PEtab observable table
         filename: Destination file name
     """
-    with open(filename, 'w') as fh:
-        df.to_csv(fh, sep='\t', index=True)
+    df.to_csv(filename, sep='\t', index=True)
 
 
 def get_output_parameters(
