@@ -17,6 +17,15 @@ from .C import *  # noqa: F403
 
 
 logger = logging.getLogger(__name__)
+__all__ = ['get_optimization_to_simulation_parameter_mapping',
+           'get_parameter_mapping_for_condition',
+           'handle_missing_overrides',
+           'merge_preeq_and_sim_pars',
+           'merge_preeq_and_sim_pars_condition',
+           'ParMappingDict', 'ParMappingDictTuple',
+           'ScaleMappingDict', 'ScaleMappingDictTuple',
+           'ParMappingDictQuadruple']
+
 
 # Parameter mapping for condition
 ParMappingDict = Dict[str, Union[str, numbers.Number]]
@@ -42,12 +51,14 @@ def get_optimization_to_simulation_parameter_mapping(
         warn_unmapped: Optional[bool] = True,
         scaled_parameters: bool = False,
         fill_fixed_parameters: bool = True,
+        allow_timepoint_specific_numeric_noise_parameters: bool = False
 ) -> List[ParMappingDictQuadruple]:
     """
     Create list of mapping dicts from PEtab-problem to SBML parameters.
 
     Mapping can be performed in parallel. The number of threads is controlled
-    by the environment variable with the name of petab.ENV_NUM_THREADS.
+    by the environment variable with the name of
+    :py:data:`petab.ENV_NUM_THREADS`.
 
     Parameters:
         condition_df, measurement_df, parameter_df, observable_df:
@@ -65,14 +76,21 @@ def get_optimization_to_simulation_parameter_mapping(
         fill_fixed_parameters:
             Whether to fill in nominal values for fixed parameters
             (estimate=0 in parameters table).
+        allow_timepoint_specific_numeric_noise_parameters:
+            Mapping of timepoint-specific parameters overrides is generally
+            not supported. If this option is set to True, this function will
+            not fail in case of timepoint-specific fixed noise parameters,
+            if the noise formula consists only of one single parameter.
+            It is expected that the respective mapping is performed elsewhere.
+            The value mapped to the respective parameter here is undefined.
 
     Returns:
         Parameter value and parameter scale mapping for all conditions.
 
         The length of the returned array is the number of unique combinations
-        of ``simulationConditionId``s and ``preequilibrationConditionId``s from
-        the measurement table. Each entry is a tuple of four dicts of length
-        equal to the number of model parameters.
+        of ``simulationConditionId`` s and ``preequilibrationConditionId`` s
+        from the measurement table. Each entry is a tuple of four dicts of
+        length equal to the number of model parameters.
         The first two dicts map simulation parameter IDs to optimization
         parameter IDs or values (where values are fixed) for preequilibration
         and simulation condition, respectively.
@@ -84,7 +102,10 @@ def get_optimization_to_simulation_parameter_mapping(
     """
 
     # Ensure inputs are okay
-    _perform_mapping_checks(measurement_df)
+    _perform_mapping_checks(
+        measurement_df,
+        allow_timepoint_specific_numeric_noise_parameters=  # noqa: E251,E501
+        allow_timepoint_specific_numeric_noise_parameters)
 
     if simulation_conditions is None:
         simulation_conditions = measurements.get_simulation_conditions(
@@ -109,7 +130,8 @@ def get_optimization_to_simulation_parameter_mapping(
             _map_condition_arg_packer(
                 simulation_conditions, measurement_df, condition_df,
                 parameter_df, sbml_model, simulation_parameters, warn_unmapped,
-                scaled_parameters, fill_fixed_parameters))
+                scaled_parameters, fill_fixed_parameters,
+                allow_timepoint_specific_numeric_noise_parameters))
         return list(mapping)
 
     # Run multi-threaded
@@ -120,29 +142,42 @@ def get_optimization_to_simulation_parameter_mapping(
             _map_condition_arg_packer(
                 simulation_conditions, measurement_df, condition_df,
                 parameter_df, sbml_model, simulation_parameters, warn_unmapped,
-                scaled_parameters, fill_fixed_parameters))
+                scaled_parameters, fill_fixed_parameters,
+                allow_timepoint_specific_numeric_noise_parameters))
     return list(mapping)
 
 
-def _map_condition_arg_packer(simulation_conditions, measurement_df,
-                              condition_df, parameter_df, sbml_model,
-                              simulation_parameters, warn_unmapped,
-                              scaled_parameters, fill_fixed_parameters):
+def _map_condition_arg_packer(
+        simulation_conditions,
+        measurement_df,
+        condition_df,
+        parameter_df,
+        sbml_model,
+        simulation_parameters,
+        warn_unmapped,
+        scaled_parameters,
+        fill_fixed_parameters,
+        allow_timepoint_specific_numeric_noise_parameters
+):
     """Helper function to pack extra arguments for _map_condition"""
     for _, condition in simulation_conditions.iterrows():
         yield(condition, measurement_df, condition_df, parameter_df,
               sbml_model, simulation_parameters, warn_unmapped,
-              scaled_parameters, fill_fixed_parameters)
+              scaled_parameters, fill_fixed_parameters,
+              allow_timepoint_specific_numeric_noise_parameters)
 
 
 def _map_condition(packed_args):
     """Helper function for parallel condition mapping.
 
-    For arguments see get_optimization_to_simulation_parameter_mapping"""
+    For arguments see
+    :py:func:`get_optimization_to_simulation_parameter_mapping`.
+    """
 
     (condition, measurement_df, condition_df, parameter_df, sbml_model,
      simulation_parameters, warn_unmapped, scaled_parameters,
-     fill_fixed_parameters) = packed_args
+     fill_fixed_parameters,
+     allow_timepoint_specific_numeric_noise_parameters) = packed_args
 
     cur_measurement_df = measurements.get_rows_for_condition(
         measurement_df, condition)
@@ -164,6 +199,8 @@ def _map_condition(packed_args):
             warn_unmapped=warn_unmapped,
             scaled_parameters=scaled_parameters,
             fill_fixed_parameters=fill_fixed_parameters,
+            allow_timepoint_specific_numeric_noise_parameters=  # noqa: E251,E501
+            allow_timepoint_specific_numeric_noise_parameters
         )
 
     par_map_sim, scale_map_sim = get_parameter_mapping_for_condition(
@@ -177,6 +214,8 @@ def _map_condition(packed_args):
         warn_unmapped=warn_unmapped,
         scaled_parameters=scaled_parameters,
         fill_fixed_parameters=fill_fixed_parameters,
+        allow_timepoint_specific_numeric_noise_parameters=  # noqa: E251,E501
+        allow_timepoint_specific_numeric_noise_parameters
     )
 
     return par_map_preeq, par_map_sim, scale_map_preeq, scale_map_sim
@@ -193,6 +232,7 @@ def get_parameter_mapping_for_condition(
         warn_unmapped: bool = True,
         scaled_parameters: bool = False,
         fill_fixed_parameters: bool = True,
+        allow_timepoint_specific_numeric_noise_parameters: bool = False,
 ) -> Tuple[ParMappingDict, ScaleMappingDict]:
     """
     Create dictionary of parameter value and parameter scale mappings from
@@ -218,18 +258,30 @@ def get_parameter_mapping_for_condition(
             Optional, saves time if precomputed.
         warn_unmapped:
             If ``True``, log warning regarding unmapped parameters
+        scaled_parameters:
+            Whether parameter values should be scaled.
         fill_fixed_parameters:
             Whether to fill in nominal values for fixed parameters
             (estimate=0 in parameters table).
+        allow_timepoint_specific_numeric_noise_parameters:
+            Mapping of timepoint-specific parameters overrides is generally
+            not supported. If this option is set to True, this function will
+            not fail in case of timepoint-specific fixed noise parameters,
+            if the noise formula consists only of one single parameter.
+            It is expected that the respective mapping is performed elsewhere.
+            The value mapped to the respective parameter here is undefined.
 
     Returns:
         Tuple of two dictionaries. First dictionary mapping model parameter IDs
         to mapped parameters IDs to be estimated or to filled-in values in case
         of non-estimated parameters.
         Second dictionary mapping model parameter IDs to their scale.
-        NaN is used where no mapping exists.
+        ``NaN`` is used where no mapping exists.
     """
-    _perform_mapping_checks(cur_measurement_df)
+    _perform_mapping_checks(
+        cur_measurement_df,
+        allow_timepoint_specific_numeric_noise_parameters=  # noqa: E251,E501
+        allow_timepoint_specific_numeric_noise_parameters)
 
     if simulation_parameters is None:
         simulation_parameters = sbml.get_model_parameters(sbml_model,
@@ -283,7 +335,7 @@ def _apply_output_parameter_overrides(
 
     Arguments:
         mapping: parameter mapping dict as obtained from
-            ``get_parameter_mapping_for_condition``
+            :py:func:`get_parameter_mapping_for_condition`.
         cur_measurement_df:
             Subset of the measurement table for the current condition
     """
@@ -317,14 +369,7 @@ def _apply_overrides_for_observable(
     """
     for i, override in enumerate(overrides):
         overridee_id = f'{override_type}Parameter{i+1}_{observable_id}'
-        try:
-            mapping[overridee_id] = override
-        except KeyError as e:
-            raise TypeError(f'Cannot override {override_type} parameter '
-                            f'{overridee_id} for observable {observable_id}.'
-                            f'Ensure there exists an {override_type} '
-                            'definition containing the correct number of '
-                            'placeholder parameters.') from e
+        mapping[overridee_id] = override
 
 
 def _apply_condition_parameters(par_mapping: ParMappingDict,
@@ -336,7 +381,7 @@ def _apply_condition_parameters(par_mapping: ParMappingDict,
     table parameter values (in-place).
 
     Arguments:
-        par_mapping: see get_parameter_mapping_for_condition
+        par_mapping: see :py:func:`get_parameter_mapping_for_condition`
         condition_id: ID of condition to work on
         condition_df: PEtab condition table
     """
@@ -370,7 +415,8 @@ def _apply_parameter_table(par_mapping: ParMappingDict,
 
     Arguments:
         par_mapping:
-            mapping dict obtained from ``get_parameter_mapping_for_condition``
+            mapping dict obtained from
+            :py:func:`get_parameter_mapping_for_condition`
         parameter_df:
             PEtab parameter table
     """
@@ -426,11 +472,17 @@ def _apply_parameter_table(par_mapping: ParMappingDict,
             scale_mapping[problem_par] = scale
 
 
-def _perform_mapping_checks(measurement_df: pd.DataFrame) -> None:
+def _perform_mapping_checks(
+        measurement_df: pd.DataFrame,
+        allow_timepoint_specific_numeric_noise_parameters: bool = False
+) -> None:
     """Check for PEtab features which we can't account for during parameter
     mapping."""
 
-    if lint.measurement_table_has_timepoint_specific_mappings(measurement_df):
+    if lint.measurement_table_has_timepoint_specific_mappings(
+            measurement_df,
+            allow_scalar_numeric_noise_parameters=  # noqa: E251,E501
+            allow_timepoint_specific_numeric_noise_parameters):
         # we could allow that for floats, since they don't matter in this
         # function and would be simply ignored
         raise ValueError(
@@ -449,7 +501,7 @@ def handle_missing_overrides(mapping_par_opt_to_par_sim: ParMappingDict,
 
     Parameters:
         mapping_par_opt_to_par_sim:
-            Output of get_parameter_mapping_for_condition
+            Output of :py:func:`get_parameter_mapping_for_condition`
         warn:
             If True, log warning regarding unmapped parameters
         condition_id:
@@ -492,10 +544,10 @@ def merge_preeq_and_sim_pars_condition(
     Arguments:
         condition_map_preeq, condition_map_sim:
             Parameter mapping as obtained from
-            `get_parameter_mapping_for_condition`
+            :py:func:`get_parameter_mapping_for_condition`
         condition_scale_map_preeq, condition_scale_map_sim:
             Parameter scale mapping as obtained from
-            `get_get_scale_mapping_for_condition`
+            :py:func:`get_parameter_mapping_for_condition`
         condition: Condition identifier for more informative error messages
     """
     if not condition_map_preeq:
@@ -564,9 +616,10 @@ def merge_preeq_and_sim_pars(
     Parameters:
         parameter_mappings:
             As returned by
-            petab.get_optimization_to_simulation_parameter_mapping
+            :py:func:`petab.get_optimization_to_simulation_parameter_mapping`.
         scale_mappings:
-            As returned by petab.get_optimization_to_simulation_scale_mapping.
+            As returned by
+            :py:func:`petab.get_optimization_to_simulation_parameter_mapping`.
 
     Returns:
         The parameter and scale simulation mappings, modified and checked.
