@@ -1,16 +1,23 @@
 """PEtab Problem class"""
 
 import os
+# Renamed to `Path_` to avoid unknown error during Sphinx doc build
+from pathlib import Path as Path_
 import tempfile
+from typing import Dict, Iterable, List, Optional, Union
 from warnings import warn
 
 import pandas as pd
 import libsbml
-from typing import Optional, List, Union, Dict, Iterable
 from . import (parameter_mapping, measurements, conditions, parameters,
                sampling, sbml, yaml, core, observables, format_version)
 from .C import *  # noqa: F403
 
+__all__ = ['Problem', 'get_default_condition_file_name',
+           'get_default_measurement_file_name',
+           'get_default_parameter_file_name',
+           'get_default_sbml_file_name']
+
 
 class Problem:
     """
@@ -233,13 +240,13 @@ class Problem:
     def from_combine(filename: str) -> 'Problem':
         """Read PEtab COMBINE archive (http://co.mbine.org/documents/archive).
 
-        See also ``create_combine_archive``.
+        See also :py:func:`petab.create_combine_archive`.
 
         Arguments:
             filename: Path to the PEtab-COMBINE archive
 
         Returns:
-            A ``petab.Problem`` instance.
+            A :py:class:`petab.Problem` instance.
         """
         # function-level import, because module-level import interfered with
         # other SWIG interfaces
@@ -264,6 +271,53 @@ class Problem:
 
         return problem
 
+    def to_files_generic(
+        self,
+        prefix_path: Union[str, Path_],
+    ) -> None:
+        """Save a PEtab problem to generic file names.
+
+        The PEtab problem YAML file is always created. PEtab data files are
+        only created if the PEtab problem contains corresponding data (e.g. a
+        PEtab visualization TSV file is only created if the PEtab problem has
+        one).
+
+        Arguments:
+            prefix_path:
+                Specify a prefix to all paths, to avoid specifying the
+                prefix for all paths individually. NB: the prefix is added to
+                paths before `relative_paths` is handled downstream in
+                `petab.yaml.create_problem_yaml`.
+
+        Returns:
+            The path to the PEtab problem YAML file.
+        """
+        prefix_path = Path_(prefix_path)
+
+        # Generate generic filenames for data tables in the PEtab problem that
+        # contain data.
+        filenames = {}
+        for table_name in [
+            'condition',
+            'measurement',
+            'parameter',
+            'observable',
+            'visualization',
+        ]:
+            if getattr(self, f'{table_name}_df') is not None:
+                filenames[f'{table_name}_file'] = f'{table_name}s.tsv'
+
+        if self.sbml_document is not None:
+            filenames['sbml_file'] = 'model.xml'
+
+        filenames['yaml_file'] = 'problem.yaml'
+
+        self.to_files(**filenames, prefix_path=prefix_path)
+
+        if prefix_path is None:
+            return filenames['yaml_file']
+        return str(prefix_path / filenames['yaml_file'])
+
     def to_files(self,
                  sbml_file: Optional[str] = None,
                  condition_file: Optional[str] = None,
@@ -271,7 +325,9 @@ class Problem:
                  parameter_file: Optional[str] = None,
                  visualization_file: Optional[str] = None,
                  observable_file: Optional[str] = None,
-                 yaml_file: Optional[str] = None) -> None:
+                 yaml_file: Optional[str] = None,
+                 prefix_path: Optional[Union[str, Path_]] = None,
+                 relative_paths: bool = True,) -> None:
         """
         Write PEtab tables to files for this problem
 
@@ -289,11 +345,34 @@ class Problem:
             visualization_file: Visualization table destination
             observable_file: Observables table destination
             yaml_file: YAML file destination
+            prefix_path:
+                Specify a prefix to all paths, to avoid specifying the
+                prefix for all paths individually. NB: the prefix is added to
+                paths before `relative_paths` is handled.
+            relative_paths:
+                whether all paths in the YAML file should be
+                relative to the location of the YAML file. If `False`, then
+                paths are left unchanged.
 
         Raises:
-            ValueError: If a destination was provided for a non-existing
-            entity.
+            ValueError:
+                If a destination was provided for a non-existing entity.
         """
+        if prefix_path is not None:
+            prefix_path = Path_(prefix_path)
+
+            def add_prefix(path0: Union[None, str, Path_]) -> str:
+                if path0 is None:
+                    return path0
+                return str(prefix_path / path0)
+
+            sbml_file = add_prefix(sbml_file)
+            condition_file = add_prefix(condition_file)
+            measurement_file = add_prefix(measurement_file)
+            parameter_file = add_prefix(parameter_file)
+            observable_file = add_prefix(observable_file)
+            visualization_file = add_prefix(visualization_file)
+            yaml_file = add_prefix(yaml_file)
 
         if sbml_file:
             if self.sbml_document is not None:
@@ -344,13 +423,14 @@ class Problem:
             yaml.create_problem_yaml(sbml_file, condition_file,
                                      measurement_file, parameter_file,
                                      observable_file, yaml_file,
-                                     visualization_file)
+                                     visualization_file,
+                                     relative_paths=relative_paths,)
 
     def get_optimization_parameters(self):
         """
         Return list of optimization parameter IDs.
 
-        See ``petab.parameters.get_optimization_parameters``.
+        See :py:func:`petab.parameters.get_optimization_parameters`.
         """
         return parameters.get_optimization_parameters(self.parameter_df)
 
@@ -358,18 +438,19 @@ class Problem:
         """
         Return list of optimization parameter scaling strings.
 
-        See ``petab.parameters.get_optimization_parameters``.
+        See :py:func:`petab.parameters.get_optimization_parameters`.
         """
         return parameters.get_optimization_parameter_scaling(self.parameter_df)
 
     def get_model_parameters(self):
-        """See `petab.sbml.get_model_parameters`"""
+        """See :py:func:`petab.sbml.get_model_parameters`"""
         return sbml.get_model_parameters(self.sbml_model)
 
     def get_observables(self, remove: bool = False):
         """
         Returns dictionary of observables definitions.
-        See `assignment_rules_to_dict` for details.
+
+        See :py:func:`petab.assignment_rules_to_dict` for details.
         """
         warn("This function will be removed in future releases.",
              DeprecationWarning)
@@ -386,6 +467,7 @@ class Problem:
         """
         Return dictionary of observableId => sigma as defined in the SBML
         model.
+
         This does not include parameter mappings defined in the measurement
         table.
         """
@@ -396,7 +478,7 @@ class Problem:
 
     def get_noise_distributions(self):
         """
-        See `get_noise_distributions`.
+        See :py:func:`petab.get_noise_distributions`.
         """
         return measurements.get_noise_distributions(
             measurement_df=self.measurement_df)
@@ -416,8 +498,7 @@ class Problem:
 
         Returns
         -------
-        v:
-            The reduced vector with applied mask.
+        The reduced vector with applied mask.
         """
         if not free and not fixed:
             return []
@@ -440,8 +521,7 @@ class Problem:
 
         Returns
         -------
-        v:
-            The parameter ids.
+        The parameter IDs.
         """
         v = list(self.parameter_df.index.values)
         return self._apply_mask(v, free=free, fixed=fixed)
@@ -478,8 +558,7 @@ class Problem:
 
         Returns
         -------
-        v:
-            The parameter nominal values.
+        The parameter nominal values.
         """
         v = list(self.parameter_df[NOMINAL_VALUE])
         if scaled:
@@ -536,8 +615,7 @@ class Problem:
 
         Returns
         -------
-        v:
-            The lower parameter bounds.
+        The lower parameter bounds.
         """
         v = list(self.parameter_df[LOWER_BOUND])
         if scaled:
@@ -572,8 +650,7 @@ class Problem:
 
         Returns
         -------
-        v:
-            The upper parameter bounds.
+        The upper parameter bounds.
         """
         v = list(self.parameter_df[UPPER_BOUND])
         if scaled:
@@ -608,7 +685,12 @@ class Problem:
         return measurements.get_simulation_conditions(self.measurement_df)
 
     def get_optimization_to_simulation_parameter_mapping(
-            self, warn_unmapped: bool = True, scaled_parameters: bool = False):
+            self,
+            warn_unmapped: bool = True,
+            scaled_parameters: bool = False,
+            allow_timepoint_specific_numeric_noise_parameters:
+            bool = False,
+    ):
         """
         See get_simulation_to_optimization_parameter_mapping.
         """
@@ -620,12 +702,15 @@ class Problem:
                 self.observable_df,
                 self.sbml_model,
                 warn_unmapped=warn_unmapped,
-                scaled_parameters=scaled_parameters)
+                scaled_parameters=scaled_parameters,
+                allow_timepoint_specific_numeric_noise_parameters=  # noqa: E251,E501
+                allow_timepoint_specific_numeric_noise_parameters
+            )
 
     def create_parameter_df(self, *args, **kwargs):
         """Create a new PEtab parameter table
 
-        See create_parameter_df
+        See :py:func:`create_parameter_df`.
         """
         return parameters.create_parameter_df(
             self.sbml_model,
@@ -637,11 +722,59 @@ class Problem:
     def sample_parameter_startpoints(self, n_starts: int = 100):
         """Create starting points for optimization
 
-        See sample_parameter_startpoints
+        See :py:func:`petab.sample_parameter_startpoints`.
         """
         return sampling.sample_parameter_startpoints(
             self.parameter_df, n_starts=n_starts)
 
+    def unscale_parameters(
+        self,
+        x_dict: Dict[str, float],
+    ) -> Dict[str, float]:
+        """Unscale parameter values.
+
+        Parameters
+        ----------
+        x_dict:
+            Keys are parameter IDs in the PEtab problem, values are scaled
+            parameter values.
+
+        Returns
+        -------
+        The unscaled parameter values.
+        """
+        return {
+            parameter_id: parameters.unscale(
+                parameter_value,
+                self.parameter_df[PARAMETER_SCALE][parameter_id],
+            )
+            for parameter_id, parameter_value in x_dict.items()
+        }
+
+    def scale_parameters(
+        self,
+        x_dict: Dict[str, float],
+    ) -> Dict[str, float]:
+        """Scale parameter values.
+
+        Parameters
+        ----------
+        x_dict:
+            Keys are parameter IDs in the PEtab problem, values are unscaled
+            parameter values.
+
+        Returns
+        -------
+        The scaled parameter values.
+        """
+        return {
+            parameter_id: parameters.scale(
+                parameter_value,
+                self.parameter_df[PARAMETER_SCALE][parameter_id],
+            )
+            for parameter_id, parameter_value in x_dict.items()
+        }
+
 
 def get_default_condition_file_name(model_name: str, folder: str = ''):
     """Get file name according to proposed convention"""
