@@ -6,12 +6,15 @@ from typing import Dict, Union, Optional, List
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
+import warnings
 
 from .helper_functions import (create_figure,
                                handle_dataset_plot,
                                check_ex_exp_columns,
                                create_or_update_vis_spec)
 
+from .plotter import MPLPlotter
+from .plotting import VisSpecParser
 from .. import problem, measurements, core, conditions
 from ..C import *
 
@@ -20,6 +23,13 @@ IdsList = List[str]
 NumList = List[int]
 
 
+__all__ = ["plot_data_and_simulation", "plot_petab_problem",
+           "plot_measurements_by_observable", "save_vis_spec",
+           "plot_with_vis_spec", "plot_without_vis_spec", "plot_problem"]
+# autodoc mixes up plot_data_and_simulation model and function and documents
+# the latter, but this function will be removed in future releases
+
+
 def plot_data_and_simulation(
         exp_data: Union[str, pd.DataFrame],
         exp_conditions: Union[str, pd.DataFrame],
@@ -90,6 +100,10 @@ def plot_data_and_simulation(
     ax: Axis object of the created plot.
     None: In case subplots are save to file
     """
+    warnings.warn("This function will be removed in future releases. "
+                  "Please use plot_with_vis_spec or plot_without_vis_spec "
+                  "instead.",
+                  DeprecationWarning)
 
     if isinstance(exp_conditions, str):
         exp_conditions = conditions.get_condition_df(exp_conditions)
@@ -112,6 +126,8 @@ def plot_data_and_simulation(
     if isinstance(exp_data, str):
         exp_data = measurements.get_measurement_df(exp_data)
     # check columns, and add non-mandatory default columns
+
+    # TODO: legend_dict is not used?
     exp_data, dataset_id_list, legend_dict = \
         check_ex_exp_columns(exp_data,
                              dataset_id_list,
@@ -135,6 +151,7 @@ def plot_data_and_simulation(
                                                    observable_num_list,
                                                    plotted_noise)
 
+    # TODO: causing some problems?
     if sim_data is not None:
         sim_data[DATASET_ID] = exp_data[DATASET_ID]
 
@@ -164,7 +181,7 @@ def plot_data_and_simulation(
             handle_dataset_plot(plot_spec, ax, exp_data,
                                 exp_conditions, sim_data)
 
-        if BAR_PLOT in vis_spec.loc[ind_plot, PLOT_TYPE_SIMULATION]:
+        if all(vis_spec.loc[ind_plot, PLOT_TYPE_SIMULATION].isin([BAR_PLOT])):
 
             legend = ['measurement']
 
@@ -207,6 +224,9 @@ def plot_petab_problem(
     Visualization using petab problem.
     For documentation, see function plot_data_and_simulation()
     """
+    warnings.warn("This function will be removed in future releases. "
+                  "Pleas use plot_problem instead",
+                  DeprecationWarning)
 
     return plot_data_and_simulation(petab_problem.measurement_df,
                                     petab_problem.condition_df,
@@ -229,7 +249,7 @@ def plot_measurements_by_observable(
     plot measurement data grouped by observable ID.
     A simple wrapper around the more complex function plot_data_and_simulation.
 
-    Parameters:
+    Parameters
     ----------
 
     data_file_path:
@@ -240,11 +260,15 @@ def plot_measurements_by_observable(
         String indicating how noise should be visualized:
         ['MeanAndSD' (default), 'MeanAndSEM', 'replicate', 'provided']
 
-    Return:
-    ----------
+    Returns
+    -------
 
     ax: axis of figures
     """
+    warnings.warn("This function will be removed in future releases. "
+                  "Please use plot_without_vis_spec without providing "
+                  "grouping_list",
+                  DeprecationWarning)
 
     # import measurement data
     measurement_data = measurements.get_measurement_df(data_file_path)
@@ -315,6 +339,9 @@ def save_vis_spec(
     output_file_path:
         File path to which  the generated visualization specification is saved.
     """
+    warnings.warn("This function will be removed in future releases. ",
+                  DeprecationWarning)
+
     # import from file in case experimental data is provided in file
     if isinstance(exp_data, str):
         exp_data = measurements.get_measurement_df(exp_data)
@@ -337,3 +364,189 @@ def save_vis_spec(
                                             plotted_noise)
 
     vis_spec.to_csv(output_file_path, sep='\t', index=False)
+
+
+def plot_with_vis_spec(
+        vis_spec_df,
+        conditions_df: Union[str, pd.DataFrame],
+        measurements_df: Optional[Union[str, pd.DataFrame]] = None,
+        simulations_df: Optional[Union[str, pd.DataFrame]] = None,
+        subplot_dir: Optional[str] = None,
+        plotter_type: str = 'mpl') -> Optional[Dict[str, plt.Subplot]]:
+    """
+    Plot measurements and/or simulations. Specification of the visualization
+    routines is provided in visualization table.
+
+    Parameters
+    ----------
+    vis_spec_df: visualization table
+    conditions_df:
+        A condition DataFrame in the PEtab format or path to the condition
+        file.
+    measurements_df:
+        A measurement DataFrame in the PEtab format or path to the data file.
+    simulations_df:
+        A simulation DataFrame in the PEtab format or path to the simulation
+        output data file.
+    subplot_dir:
+        A path to the folder where single subplots should be saved.
+        PlotIDs will be taken as file names.
+    plotter_type:
+        Specifies which library should be used for plot generation. Currently,
+        only matplotlib is supported.
+
+    Returns
+    -------
+    ax: Axis object of the created plot.
+    None: In case subplots are saved to a file.
+    """
+
+    if measurements_df is None and simulations_df is None:
+        raise TypeError('Not enough arguments. Either measurements_data '
+                        'or simulations_data should be provided.')
+
+    vis_spec_parser = VisSpecParser(conditions_df, measurements_df,
+                                    simulations_df)
+    figure, dataprovider = vis_spec_parser.parse_from_vis_spec(
+        vis_spec_df)
+
+    if plotter_type == 'mpl':
+        plotter = MPLPlotter(figure, dataprovider)
+    else:
+        raise NotImplementedError('Currently, only visualization with '
+                                  'matplotlib is possible.')
+    ax = plotter.generate_figure(subplot_dir)
+    return ax
+
+
+def plot_without_vis_spec(
+        conditions_df: Union[str, pd.DataFrame],
+        grouping_list: Optional[List[IdsList]] = None,
+        group_by: str = 'observable',
+        measurements_df: Optional[Union[str, pd.DataFrame]] = None,
+        simulations_df: Optional[Union[str, pd.DataFrame]] = None,
+        plotted_noise: str = MEAN_AND_SD,
+        subplot_dir: Optional[str] = None,
+        plotter_type: str = 'mpl'
+) -> Optional[Dict[str, plt.Subplot]]:
+    """
+    Plot measurements and/or simulations. What exactly should be plotted is
+    specified in a grouping_list.
+    If grouping list is not provided, measurements (simulations) will be
+    grouped by observable, i.e. all measurements for each observable will be
+    visualized on one plot.
+
+    Parameters
+    ----------
+    grouping_list:
+        A list of lists. Each sublist corresponds to a plot, each subplot
+        contains the Ids of datasets or observables or simulation conditions
+        for this plot.
+    group_by:
+        Grouping type.
+        Possible values: 'dataset', 'observable', 'simulation'
+    conditions_df:
+        A condition DataFrame in the PEtab format or path to the condition
+        file.
+    measurements_df:
+        A measurement DataFrame in the PEtab format or path to the data file.
+    simulations_df:
+        A simulation DataFrame in the PEtab format or path to the simulation
+        output data file.
+    plotted_noise:
+        A string indicating how noise should be visualized:
+        ['MeanAndSD' (default), 'MeanAndSEM', 'replicate', 'provided']
+    subplot_dir:
+        A path to the folder where single subplots should be saved.
+        PlotIDs will be taken as file names.
+    plotter_type:
+        Specifies which library should be used for plot generation. Currently,
+        only matplotlib is supported
+
+    Returns
+    -------
+    ax: Axis object of the created plot.
+    None: In case subplots are saved to a file.
+    """
+
+    if measurements_df is None and simulations_df is None:
+        raise TypeError('Not enough arguments. Either measurements_data '
+                        'or simulations_data should be provided.')
+
+    vis_spec_parser = VisSpecParser(conditions_df, measurements_df,
+                                    simulations_df)
+
+    figure, dataprovider = vis_spec_parser.parse_from_id_list(
+        grouping_list, group_by, plotted_noise)
+
+    if plotter_type == 'mpl':
+        plotter = MPLPlotter(figure, dataprovider)
+    else:
+        raise NotImplementedError('Currently, only visualization with '
+                                  'matplotlib is possible.')
+    ax = plotter.generate_figure(subplot_dir)
+    return ax
+
+
+def plot_problem(petab_problem: problem.Problem,
+                 simulations_df: Optional[Union[str, pd.DataFrame]] = None,
+                 grouping_list: Optional[List[IdsList]] = None,
+                 group_by: str = 'observable',
+                 plotted_noise: str = MEAN_AND_SD,
+                 subplot_dir: Optional[str] = None,
+                 plotter_type: str = 'mpl'
+                 ) -> Optional[Dict[str, plt.Subplot]]:
+    """
+    Visualization using petab problem.
+    If Visualization table is part of the petab_problem, it will be used for
+    visualization. Otherwise, grouping_list will be used.
+    If neither Visualization table nor grouping_list are available,
+    measurements (simulations) will be grouped by observable, i.e. all
+    measurements for each observable will be visualized on one plot.
+
+    Parameters
+    ----------
+    petab_problem:
+        A PEtab problem
+    simulations_df:
+        A simulation DataFrame in the PEtab format or path to the simulation
+        output data file.
+    grouping_list:
+        A list of lists. Each sublist corresponds to a plot, each subplot
+        contains the Ids of datasets or observables or simulation conditions
+        for this plot.
+    group_by:
+        Possible values: 'dataset', 'observable', 'simulation'
+    plotted_noise:
+        A string indicating how noise should be visualized:
+        ['MeanAndSD' (default), 'MeanAndSEM', 'replicate', 'provided']
+    subplot_dir:
+        A string which is taken as path to the folder where single subplots
+        should be saved. PlotIDs will be taken as file names.
+    plotter_type:
+        Specifies which library should be used for plot generation. Currently,
+        only matplotlib is supported
+
+    Returns
+    -------
+    ax: Axis object of the created plot.
+    None: In case subplots are saved to a file.
+    """
+
+    if petab_problem.visualization_df is not None:
+        ax = plot_with_vis_spec(petab_problem.visualization_df,
+                                petab_problem.condition_df,
+                                petab_problem.measurement_df,
+                                simulations_df,
+                                subplot_dir,
+                                plotter_type)
+    else:
+        ax = plot_without_vis_spec(petab_problem.condition_df,
+                                   grouping_list,
+                                   group_by,
+                                   petab_problem.measurement_df,
+                                   simulations_df,
+                                   plotted_noise,
+                                   subplot_dir,
+                                   plotter_type)
+    return ax
